#
# $Id: State.pm,v 0.1 2001/04/25 10:41:48 ram Exp $
#
#  Copyright (c) 2000-2001, Raphael Manfredi
#  
#  You may redistribute only under the terms of the Artistic License,
#  as specified in the README file that comes with the distribution.
#
# HISTORY
# $Log: State.pm,v $
# Revision 0.1  2001/04/25 10:41:48  ram
# Baseline for first Alpha release.
#
# $EndLog$
#

use strict;

package Pod::PP::State;

use Carp::Datum;
use Log::Agent;

use Pod::PP::State::Info;

#
# ->make
#
# Creation routine
#
sub make {
	my $self = bless [], shift;
	return $self;
}

#
# ->push
#
# Add nested branch.
#
sub push {
	my $self = shift;
	my ($cmd, $state, $podinfo) = @_;

	my $sinfo = Pod::PP::State::Info->make($cmd, $state, $podinfo);
	push(@$self, $sinfo);

	return;
}

#
# ->pop
#
# Leaving nested branch.
#
sub pop {
	my $self = shift;
	my ($podinfo) = @_;

	unless (@$self) {
		my ($file, $line) = $podinfo->file_line();
		logwarn "improper Pod::PP 'endif' ('if' missing?) at \"%s\", line %d",
			$file, $line;
		return;
	}

	pop @$self;
	return;
}

#
# ->state
#
# Return current state
#
sub state {
	my $self = shift;

	return POD_PP_STATE_OK unless @$self;
	return $self->[-1]->state;		# State of latest item
}

#
# ->replace
#
# Return state of topmost element.
#
sub replace {
	my $self = shift;
	my ($cmd, $state, $podinfo) = @_;

	unless (@$self) {
		my ($file, $line) = $podinfo->file_line();
		logwarn "improper Pod::PP '%s' ('if' missing?) at \"%s\", line %d",
			$cmd, $file, $line;
		return;
	}

	$self->[-1]->replace($cmd, $state, $podinfo);

	return;
}

#
# ->pending
#
# Returns ($cmd, $podinfo) of last branch taken.
# Returns () when stack is empty, indicating no pending branch.
#
sub pending {
	my $self = shift;

	return () unless @$self;

	my $top = $self->[-1];
	return ($top->cmd, $top->podinfo);
}

#
# ->reset
#
# Empty stack, warning them if there are pending branches.
#
sub reset {
	my $self = shift;

	my ($cmd, $podinfo) = $self->pending;
	if (defined $cmd) {
		my ($file, $line) = $podinfo->file_line;
		logwarn "unclosed Pod::PP '%s' directive (started at \"%s\", line %d)",
			$cmd, $file, $line;
	}

	@$self = ();

	return;
}

1;

